@@ -1,5 +1,5 @@
 package Terse;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 use 5.006;
 use strict;
 use warnings;
@@ -48,7 +48,7 @@ sub run {
 
 	$j->request = Plack::Request->new($args{plack_env});
 	$j->response = $pkg->new(
-		authenticated => 0,
+		authenticated => \0,
 		error => \0,
 		errors => [],
 	);
@@ -58,6 +58,8 @@ sub run {
 		return $j->_response($j->response);
 	}
 
+	$j->{_application_has_dispatcher} = !! $j->{_application}->can('dispatch');
+
 	my $content_type = $j->request->content_type;
 	if ($content_type && $content_type =~ m/application\/json/) {
 		$j->graft('params', $j->request->raw_body || "{}");
@@ -83,7 +85,7 @@ sub run {
 	}
 
 	my $auth = $j->{_auth};
-	my $session = $j->{_application}->$auth($j, $pkg->new());
+	my ($session) = $j->_dispatch($auth, $pkg->new());
 	my $req = $j->params->req;
 	$req =~ /^([a-z][0-9a-zA-Z_]{1,31})$/; $req = $1 // '';
 	if (!$req || !$session || $PRIVATE{$req}) {
@@ -94,24 +96,18 @@ sub run {
 	$j->response->authenticated = \1;
 	$j->session = $session;
 
-	unless ($j->{_application}->can($req)) {
-		$j->response->raiseError('Invalid request', 400);
-		return $j->_response($j->response);
-	}
-
 	my ($login, $logout) = (
 		$j->{_login} eq $req,
 		$j->{_logout} eq $req
 	);
 
-	my $out = eval { $j->{_application}->$req($j) };
-	if ($@) {
-		$j->response->raiseError(['Unclassified Error', $@], 400);
-		return $j->_response($j->response);
-	}
+	my ($out) = $j->_dispatch($req); 
+	
+	return $j->_response($j->response) if $j->response->error;
+
 	$j->session = $out if ( $login || $logout );
 
-	$j->session = $j->{_application}->$auth($j, $j->session) if $j->response->authenticated;
+	($j->session) = $j->_dispatch($auth, $j, $j->session)  if $j->response->authenticated;
 
 	if ((!$j->response->authenticated || !$j->session) && !($login || $logout)) {
 		$j->response->raiseError('Unauthenticated during the request', 400);
@@ -133,10 +129,32 @@ sub _response {
 	my $res = $self->request->new_response($response_body->status_code ||= $status ||= 200);
 	$res->cookies->{sid} = $cookie if $cookie;
 	$res->content_type('application/json');
+	for (qw/error authenticated/) {
+		$response_body->$_ = $response_body->$_ ? \1 : \0 
+			if exists $response_body->{$_};
+	}
 	$res->body($response_body->serialize());
 	return $res->finalize;
 }
 
+sub _dispatch {
+	my ($self, $method, @params) = @_;
+	my @out = $self->{_application_has_dispatcher} ? eval {
+		$self->{_application}->dispatch($method, $self, @params)
+	} : eval {
+		unless ($self->{_application}->can($method)) {
+			$self->response->raiseError('Invalid request', 400);
+			return;
+		}
+		$self->{_application}->$method($self, @params);
+	};
+	if ($@) {
+		$self->response->raiseError(['Error while dispatching the request', $@], 400);
+		return;
+	}
+	return @out;
+}
+
 sub logger {
 	my ($self, $logger) = @_;
 	$self->{_logger} = $logger if ($logger);
@@ -240,8 +258,8 @@ sub AUTOLOAD : lvalue {
 	my $miss = Want::want('REF OBJECT') ? {} : '';
 	my $retval = $_[0]->{$key};
 	return $retval->(@_) if (ref $retval eq 'CODE');
-	my $isBool = Want::want('SCALAR BOOL') && ((reftype($retval) // '') eq 'SCALAR');
-	$retval = $$retval if $isBool;
+	#my $isBool = Want::want('SCALAR BOOL') && ((reftype($retval) // '') eq 'SCALAR');
+	#$retval = $$retval if $isBool;
 	$_[0]->{$key} = $_[1] // $retval // $miss;
 	$_[0]->_bless_tree($_[0]->{$key}) if ref $_[0]->{$key} eq 'HASH' || ref $_[0]->{$key} eq 'ARRAY';
 	$_[0]->{$key};
